# -*- coding: utf-8 -*-

# zope imports
from Products.ATContentTypes import ATCTMessageFactory as ATMF
from collective.z3cform.widgets.enhancedtextlines import (
    EnhancedTextLinesFieldWidget,
)
from plone.app.textfield import RichText
from plone.app.textfield.widget import RichTextFieldWidget
from plone.autoform import directives
from plone.dexterity.content import Item
from plone.supermodel import model
from zope import schema
from zope.interface import implementer

# local import
from ps.plone.mls import _, interfaces


class IFeaturedListings(model.Schema):
    """A list of featured MLS Listings."""

    directives.widget(listing_ids=EnhancedTextLinesFieldWidget)
    listing_ids = schema.List(
        description=_(u'Add one Listing ID for each entry to show up.'),
        title=_(u'MLS Listing IDs'),
        unique=True,
        value_type=schema.TextLine(
            title=_(u'ID'),
        ),
    )

    directives.widget(body_text=RichTextFieldWidget)
    body_text = RichText(
        required=False,
        title=ATMF(u'label_body_text', default=u'Body Text'),
    )


@implementer(
    IFeaturedListings,
    interfaces.IListingTraversable,
)
class FeaturedListings(Item):
    """A list of featured MLS Listings."""



# -*- coding: utf-8 -*-
"""Propertyshelf MLS Plone Embedding."""

# python imports
import logging

# zope imports
from zope.i18nmessageid import MessageFactory

# local imports
from ps.plone.mls import config

logger = logging.getLogger(config.PROJECT_NAME)
_ = MessageFactory('ps.plone.mls')


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



# -*- coding: utf-8 -*-
"""Test Layer for ps.plone.mls."""

# zope imports
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
    PLONE_FIXTURE,
)
from plone.testing import Layer, z2


class Fixture(PloneSandboxLayer):
    """Custom Test Layer for ps.plone.mls."""
    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        """Set up Zope for testing."""
        # Load ZCML
        import ps.plone.mls
        self.loadZCML(package=ps.plone.mls)

    def setUpPloneSite(self, portal):
        """Set up a Plone site for testing."""
        self.applyProfile(portal, 'ps.plone.mls:default')
        self.applyProfile(portal, 'ps.plone.mls:testfixture')


FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE, ),
    name='ps.plone.mls:Integration',
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE, z2.ZSERVER_FIXTURE),
    name='ps.plone.mls:Functional',
)

ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name='ps.plone.mls:Acceptance')

ROBOT_TESTING = Layer(name='ps.plone.mls:Robot')



# -*- coding: utf-8 -*-
"""Variables used by plone.app.robotframework."""

# python imports
import pkg_resources


CMFPLONE_VERSION = pkg_resources.get_distribution('Products.CMFPlone').version

if CMFPLONE_VERSION.startswith('5.'):
    PSPLONEMLS_PLONE_SELECTORS = 'selectors/plone-5.x.robot'
elif CMFPLONE_VERSION.startswith('4.3'):
    PSPLONEMLS_PLONE_SELECTORS = 'selectors/plone-4.3.x.robot'
elif CMFPLONE_VERSION.startswith('4.2'):
    PSPLONEMLS_PLONE_SELECTORS = 'selectors/plone-4.2.x.robot'
elif CMFPLONE_VERSION.startswith('4.1'):
    PSPLONEMLS_PLONE_SELECTORS = 'selectors/plone-4.1.x.robot'
elif CMFPLONE_VERSION.startswith('4.'):
    PSPLONEMLS_PLONE_SELECTORS = 'selectors/plone-4.x.robot'

PSPLONEMLS_DEFAULT_SELECTORS = 'selectors/default.robot'



# -*- coding: utf-8 -*-
"""Vocabularies for development projects."""

# zope imports
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

# local imports
from ps.plone.mls import _


@implementer(IVocabularyFactory)
class SortOptionsVocabulary(object):

    def __call__(self, context):
        items = []
        items.append(SimpleTerm('created', _(u'Creation Date')))
        items.append(SimpleTerm('sortable_title', _(u'Title')))
        return SimpleVocabulary(items)

SortOptionsVocabularyFactory = SortOptionsVocabulary()



# -*- coding: utf-8 -*-
"""Zope2 install and uninstall."""

# local imports
from ps.plone.mls import (
    config,
    logger,
)


def uninstall(portal, reinstall=False):
    """Uninstall the add-on."""
    if not reinstall:
        setup_tool = portal.portal_setup
        setup_tool.runAllImportStepsFromProfile(config.UNINSTALL_PROFILE)
        logger.info('Uninstall done')



# -*- coding: utf-8 -*-
"""Setup for ps.plone.mls package."""

from setuptools import setup, find_packages

version = '0.5.1'
description = "Propertyshelf MLS Plone Embedding."
long_description = ('\n'.join([
    open('README.rst').read(),
    open('CHANGES.rst').read(),
]))

install_requires = [
    'setuptools',
    # -*- Extra requirements: -*-
    'collective.z3cform.widgets',
    'mls.apiclient',
    'plone.api',
    'plone.app.dexterity',
    'plone.mls.listing >= 0.9.11',
    'ps.plone.realestatefont',
]

setup(
    name='ps.plone.mls',
    version=version,
    description=description,
    long_description=long_description,
    # Get more strings from
    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.1",
        "Framework :: Plone :: 4.2",
        "Framework :: Plone :: 4.3",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='plone zope mls listing real estate',
    author='Propertyshelf, Inc.',
    author_email='development@propertyshelf.com',
    url='https://github.com/propertyshelf/ps.plone.mls',
    download_url='http://pypi.python.org/pypi/ps.plone.mls',
    license='GPL',
    packages=find_packages('src', exclude=['ez_setup']),
    package_dir={'': 'src'},
    namespace_packages=['ps', 'ps.plone'],
    include_package_data=True,
    zip_safe=False,
    extras_require=dict(
        test=[
            'httpretty',
            'plone.app.robotframework',
            'plone.app.testing',
            'plone.mls.listing[fotorama]',
            'robotframework-selenium2screenshots',
        ],
    ),
    install_requires=install_requires,
    entry_points="""
    # -*- Entry points: -*-

    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
"""MLS integration api."""

# python imports
from mls.apiclient import (
    api,
    exceptions,
    resources,
)

# zope imports
from plone import api as plone_api
from plone.mls.core.api import get_settings
from plone.mls.listing.api import get_agency_info
from zope.annotation.interfaces import IAnnotations
from zope.globalrequest import getRequest

# local imports
from ps.plone.mls import (
    _,
    utils,
)

MIN_MAX_FIELDS = [
    'baths',
    'beds',
    'lot_size',
    'interior_area',
]


def _remove_omitted(params, omit):
    """Removed omitted keys from the params dict."""
    if omit is not None and isinstance(omit, (list, tuple)):
        for item in omit:
            try:
                del params[item]
            except KeyError:
                continue


def prepare_search_params(params, context=None, omit=None):
    """Prepare search params."""
    settings = get_settings(context=context)
    result = {}

    _remove_omitted(params, omit)

    for item in params:
        if item in MIN_MAX_FIELDS:
            min_max = params[item]
            if isinstance(min_max, (list, tuple, )):
                if len(min_max) > 0 and min_max[0] != '--MINVALUE--':
                    result[item + '_min'] = min_max[0]
                if len(min_max) > 1 and min_max[1] != '--MAXVALUE--':
                    result[item + '_max'] = min_max[1]
                continue

        # Convert lists and tuples to comma separated lists.
        if isinstance(params[item], (list, tuple, )):
            if len(params.get(item, ())) > 0:
                params[item] = ','.join(params[item])
            else:
                params[item] = None

        # Remove all None-Type values.
        if params[item] is not None:
            value = params[item]
            if isinstance(value, unicode):
                value = value.encode('utf-8')
            result[item] = value

    agency_developments = result.pop('agency_developments', False)
    if agency_developments is True:
        result['agency_developments'] = settings.get('agency_id', None)

    return result


def get_api(context=None, lang=None):
    """Get the API Client based on the local configuration."""
    settings = get_settings(context=context)
    base_url = settings.get('mls_site', None)
    api_key = settings.get('mls_key', None)
    debug = plone_api.env.debug_mode
    mls = api.API(base_url, api_key=api_key, lang=lang, debug=debug)
    return mls


def get_development(item_id=None, context=None, request=None, lang=None):
    """Get a single development."""
    mlsapi = get_api(context=context, lang=lang)
    try:
        item = Development.get(mlsapi, item_id)
    except exceptions.ServerError:
        if not plone_api.user.is_anonymous():
            message = _(
                u'An error occured trying to connect to the '
                u'configured MLS. Please check your settings or try '
                u'again later.'
            )
            plone_api.portal.show_message(
                message=message,
                request=request,
                type='warning',
            )
    except exceptions.ResourceNotFound:
        message = _(
            u'The requested development was not found.'
        )
        plone_api.portal.show_message(
            message=message,
            request=request,
            type='info',
        )
    except exceptions.ConnectionError:
        pass
    else:
        return item


class Field(object):
    """Wraps the api data into a field structure."""

    def __init__(self, name, value, resource, title=None):
        self.name = name
        self.value = value
        self.title = title

        if self.title is not None:
            return

        # Try to get the correct label for the field.
        titles = resource.field_titles().get('response', {})

        try:
            self.title = titles['fields'][name]
        except (KeyError, TypeError):
            pass
        else:
            return

        group_fields = titles.get('group_fields', [])
        for group in group_fields:
            try:
                self.title = group_fields[group][name]
            except (KeyError, TypeError):
                continue
            else:
                break
        if self.title is None:
            self.title = name


class CacheMixin(object):
    """Extend API resources to handle some caching."""
    _field_titles = None

    def _get_field_titles(self):
        """Get the translated field titles from the API endpoint."""
        return self.__class__.get_field_titles(self._api)

    def field_titles(self):
        """Get the translated field titles for that resource."""
        if self._field_titles is not None:
            return self._field_titles

        request = getRequest()
        key = 'cache-{0}-{1}-{2}'.format(
            self.__class__.__name__,
            self._api.base_url,
            self._api.lang,
        )
        cache = IAnnotations(request)
        data = cache.get(key, None)
        if not data:
            try:
                data = self._get_field_titles()
            except exceptions.ResourceNotFound:
                data = {}
            else:
                cache[key] = data
                self._field_titles = data
        else:
            self._field_titles = data
        return data

    def __getattr__(self, name):
        """Returns a data attribute or raises AttributeError.

        This version wraps the return value into a Field class for better
        access in Plone.
        """
        try:
            value = self._data[name]
        except KeyError:
            return object.__getattribute__(self, name)
        else:
            if value is not None:
                return Field(name, value, self)


class Agency(CacheMixin, resources.Agency):
    """'Agency' entity resource class with caching support."""

    def override(self, context=None):
        """Override the agency information with local settings."""
        settings = get_agency_info(context=context)
        if settings is None:
            return

        mls_settings = get_settings(context=context)
        agency_id = mls_settings.get('agency_id', None)

        if self._data.get('id', None) == agency_id:
            if settings and settings.get('force', False) is True:
                pass
            else:
                return

        mapping = {
            'agency_address': 'address',
            'agency_description': 'description',
            'agency_email': 'email',
            'agency_email_alternative': 'email_alternative',
            'agency_geo_location': 'geo_location',
            'agency_logo_url': 'logo',
            'agency_name': 'title',
            'agency_office_fax': 'office_fax',
            'agency_office_phone': 'office_phone',
            'agency_office_phone_alternative': 'office_phone_alternative',
            'agency_website': 'website',
        }

        utils.merge_local_contact_info(
            settings=settings,
            mapping=mapping,
            data=self._data,
        )


class Agent(CacheMixin, resources.Agent):
    """'Agent' entity resource class with caching support."""

    def override(self, context=None):
        """Override the agent information with local settings."""
        settings = get_agency_info(context=context)
        if settings is None:
            return

        mls_settings = get_settings(context=context)
        agency_id = mls_settings.get('agency_id', None)

        if self._data.get('id', None) == agency_id:
            if settings and settings.get('force', False) is True:
                pass
            else:
                return

        mapping = {
            'agent_avatar_url': 'avatar',
            'agent_cell_phone': 'cell_phone',
            'agent_email': 'email',
            'agent_fax': 'fax',
            'agent_name': 'name',
            'agent_office_phone': 'office_phone',
            'agent_title': 'user_title',
        }

        utils.merge_local_contact_info(
            settings=settings,
            mapping=mapping,
            data=self._data,
        )


class Development(CacheMixin, resources.Development):
    """'Development Project' entity resource class with caching support."""

    def __init__(self, api, data):
        super(Development, self).__init__(api, data)
        self.__class_agency__ = Agency
        self.__class_agent__ = Agent
        self.__class_group__ = PropertyGroup
        self.__class_phase__ = DevelopmentPhase


class DevelopmentPhase(CacheMixin, resources.DevelopmentPhase):
    """'Development Phase' entity resource class with caching support."""


class Listing(CacheMixin, resources.Listing):
    """'Listing' entity resource class with caching support."""


class PropertyGroup(CacheMixin, resources.PropertyGroup):
    """'Property Group' entity resource class with caching support."""



# -*- coding: utf-8 -*-
"""MLS development collection."""

# zope imports
from plone.app.layout.viewlets.common import ViewletBase
from plone.directives import form
from plone.memoize.view import memoize
from plone.mls.core.navigation import ListingBatch
from z3c.form import button
from plone.supermodel.directives import fieldset
# from z3c.form.browser import checkbox
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.component import queryMultiAdapter
from zope.interface import (
    alsoProvides,
    noLongerProvides,
)
from zope.traversing.browser.absoluteurl import absoluteURL

# local imports
from plone.mls.listing.browser.interfaces import (
    IBaseListingItems,
    IListingDetails,
)
from ps.plone.mls import (
    _,
    api,
    config,
)
from ps.plone.mls.interfaces import (
    IDevelopmentCollection,
    IDevelopmentDetails,
    IPossibleDevelopmentCollection,
)


#: Reduce the API load by defining the fields we need.
FIELDS = [
    'id',
    'title',
    'logo',
    'location',
    'lot_size',
    'location_type',
    'number_of_listings',
    'number_of_groups',
]

EXCLUDED_SEARCH_FIELDS = [
    'map_zoom_level',
    'show_contact_form',
    'contact_override',
    'show_contact_link',
    'show_contact_info',
    'show_captcha',
    'modify_urls',
    'limit_developments',
    'limit_listings',
]


class DevelopmentCollectionViewlet(ViewletBase):
    """Dynamic collection of MLS developments."""

    _items = None
    _batching = None
    limit = None

    @property
    def available(self):
        """Check if this viewlet is available for rendering."""
        return IDevelopmentCollection.providedBy(self.context) and \
            not IDevelopmentDetails.providedBy(self.view) and \
            not IBaseListingItems.providedBy(self.view) and \
            not IListingDetails.providedBy(self.view)

    @property
    def config(self):
        """Get the embedding configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_DEVELOPMENT_COLLECTION, {})

    def update(self):
        """Prepare view related data."""
        super(DevelopmentCollectionViewlet, self).update()
        self.context_state = queryMultiAdapter(
            (self.context, self.request), name='plone_context_state',
        )

        if self.available:
            # Only query the MLS if we show the viewlet.
            self.limit = self.config.get('limit_developments', 5)
            self._get_items()

    @property
    @memoize
    def items(self):
        """Return the collection items."""
        return self._items

    def _get_items(self):
        """Get the collection items from the MLS."""
        lang = self.portal_state.language()
        mlsapi = api.get_api(context=self.context, lang=lang)
        params = {
            # 'summary': '1',
            'fields': u','.join(FIELDS),
            'limit': self.limit,
            'offset': self.request.get('b_start', 0),
        }
        params.update(self.config)
        params = api.prepare_search_params(
            params,
            context=self.context,
            omit=EXCLUDED_SEARCH_FIELDS,
        )
        try:
            result = api.Development.search(mlsapi, params=params)
        except Exception:
            pass
        else:
            self._items = result.get_items()
            headers = result.get_headers()
            self._batching = {
                'results': headers.get('CountTotal'),
            }

    @memoize
    def view_url(self):
        """Generate view url."""
        if not self.context_state.is_view_template():
            # We have the default view rendered.
            return self.context_state.current_base_url()
        else:
            return '/'.join([
                absoluteURL(self.context, self.request),
                '',
            ])

    @property
    def batching(self):
        """Create the batch provider for the collection items."""
        if not self.items:
            return
        return ListingBatch(
            self.items,
            self.limit,
            self.request.get('b_start', 0),
            orphan=1,
            batch_data=self._batching,
        )

    def get_url(self, item):
        """Get the (possibly modified) URL for the development item."""
        url = u'{0}{1}'.format(self.view_url(), item.id.value)
        if self.config.get('modify_url', True):
            url = u'{0}___{1}-{2}'.format(
                url,
                item.title.value,
                item.location.value,
            )
        return url


FIELDS_FILTER = (
    'agency_developments',
    'q',
    'sort_on',
    'reverse',
)


class IDevelopmentCollectionConfiguration(form.Schema):
    """Development Collection Configuration Form."""

    fieldset(
        'filter',
        label=_(u'Filter Options'),
        fields=FIELDS_FILTER,
    )

    # Fields for 'Defautl' fieldset.
    map_zoom_level = schema.Int(
        default=7,
        description=_(
            u'Set the resolution for the map. 0 is the lowest zoom level, in '
            u'which the entire world can be seen. The higher the zoom level, '
            u'the more details can be seen. The available maximum zoom level '
            u'value differs from area to area and can change over time, also '
            u'depending on the used map provider.'
        ),
        min=0,
        max=21,
        required=True,
        title=_(u'Zoom level for maps'),
    )

    show_contact_info = schema.Bool(
        default=False,
        description=_(
            u'If enabled, the contact information for a development will be '
            u'shown on the detail page for a development.'
        ),
        required=False,
        title=_(u'Show Contact Information'),
    )

    show_contact_form = schema.Bool(
        default=False,
        description=_(
            u'If enabled, a form to contact the responsible agent will be '
            u'shown on the detail page for a development.'
        ),
        required=False,
        title=_(u'Show Contact Form'),
    )

    contact_override = schema.TextLine(
        description=_(
            u'Send the contact form to this email instead direct to the agent'
        ),
        required=False,
        title=_(u'Alternative Email recipient'),
    )

    show_contact_link = schema.Bool(
        default=False,
        description=_(
            u'If the contact form is enabled, an anchor link will show as '
            u'quick navigation to the form.'
        ),
        required=False,
        title=_(u'Show Contact-Us anchor link'),
    )

    show_captcha = schema.Bool(
        default=False,
        description=_(
            u'Show a captcha field within the contact form to prevent spam.'
        ),
        required=False,
        title=_(u'Show Captcha'),
    )

    modify_url = schema.Bool(
        default=True,
        description=_(
            u'Modify the individual development URLs to show extra '
            u'information, such as the Title and Location'
        ),
        required=False,
        title=_(u'Modify URLs'),
    )

    limit_developments = schema.Int(
        default=5,
        description=_(
            u'How many developments should be shown per page?'
        ),
        required=True,
        title=_(u'Developments per Page'),
    )

    limit_listings = schema.Int(
        default=25,
        description=_(
            u'How many listings should be shown per devlopment listings page?'
        ),
        required=True,
        title=_(u'Listings per Page'),
    )

    # Fields for 'Filter Options' fieldset.
    agency_developments = schema.Bool(
        default=False,
        description=_(
            u'If enabled, only developments for the configured agency '
            u'will be shown.'
        ),
        required=False,
        title=_(u'Agency Developments'),
    )

    q = schema.TextLine(
        description=_(
            u'Enter a search term to filter the results.'
        ),
        required=False,
        title=_(u'Searchable Text'),
    )

    sort_on = schema.Choice(
        description=_(
            u'How should the results be sorted? If nothing is selected, the '
            u'results are sorted by relevance.'
        ),
        required=False,
        title=_('Sort results by'),
        vocabulary='ps.plone.mls.developments.sort_options',
    )

    reverse = schema.Bool(
        default=False,
        required=False,
        title=_('Reverse sort order?'),
    )

    # location_state = schema.Choice(
    #     required=False,
    #     title=_(u'State'),
    #     source='plone.mls.listing.LocationStates',
    # )

    # location_county = schema.Choice(
    #     required=False,
    #     title=_(u'County'),
    #     source='plone.mls.listing.LocationCounties',
    # )

    # location_district = schema.Choice(
    #     required=False,
    #     title=_(u'District'),
    #     source='plone.mls.listing.LocationDistricts',
    # )

    # location_type = schema.Tuple(
    #     required=False,
    #     title=_(u'Location Type'),
    #     value_type=schema.Choice(
    #         source='plone.mls.listing.LocationTypes'
    #     ),
    # )

    # geographic_type = schema.Tuple(
    #     required=False,
    #     title=_(u'Geographic Type'),
    #     value_type=schema.Choice(
    #         source='plone.mls.listing.GeographicTypes'
    #     ),
    # )

    # view_type = schema.Tuple(
    #     required=False,
    #     title=_(u'View Type'),
    #     value_type=schema.Choice(
    #         source='plone.mls.listing.ViewTypes'
    #     ),
    # )


class DevelopmentCollectionConfiguration(form.SchemaForm):
    """Development Collection Configuration Form."""

    schema = IDevelopmentCollectionConfiguration
    label = _(u'\'Development Collection\' Configuration')
    description = _(
        u'Adjust the behaviour for this \'Development Collection\' viewlet.'
    )

    def getContent(self):
        annotations = IAnnotations(self.context)
        return annotations.get(
            config.SETTINGS_DEVELOPMENT_COLLECTION,
            annotations.setdefault(config.SETTINGS_DEVELOPMENT_COLLECTION, {})
        )

    @button.buttonAndHandler(_(u'Save'))
    def handle_save(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        annotations = IAnnotations(self.context)
        annotations[config.SETTINGS_DEVELOPMENT_COLLECTION] = data
        self.request.response.redirect(absoluteURL(self.context, self.request))

    @button.buttonAndHandler(_(u'Cancel'))
    def handle_cancel(self, action):
        self.request.response.redirect(absoluteURL(self.context, self.request))


class DevelopmentCollectionStatus(object):
    """Return activation/deactivation status of the viewlet."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def can_activate(self):
        return IPossibleDevelopmentCollection.providedBy(self.context) and \
            not IDevelopmentCollection.providedBy(self.context)

    @property
    def active(self):
        return IDevelopmentCollection.providedBy(self.context)


class DevelopmentCollectionToggle(object):
    """Toggle DevelopmentCollection viewlet for the current context."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        msg_type = 'info'

        if IDevelopmentCollection.providedBy(self.context):
            # Deactivate DevelopmentCollection viewlet.
            noLongerProvides(self.context, IDevelopmentCollection)
            self.context.reindexObject(idxs=['object_provides', ])
            msg = _(u'\'Development Collection\' viewlet deactivated.')
        elif IPossibleDevelopmentCollection.providedBy(self.context):
            alsoProvides(self.context, IDevelopmentCollection)
            self.context.reindexObject(idxs=['object_provides', ])
            msg = _(u'\'Development Collection\' viewlet activated.')
        else:
            msg = _(
                u'The \'Development Collection\' viewlet does\'t work with '
                u'this content type. Add \'IPossibleDevelopmentCollection\' '
                u'to the provided interfaces to enable this feature.'
            )
            msg_type = 'error'

        self.context.plone_utils.addPortalMessage(msg, msg_type)
        self.request.response.redirect(self.context.absolute_url())
